package jp.ne.xuni.seleneserver;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Properties;

public class SeleneServer
	extends UnicastRemoteObject implements ISeleneServer {
	public static final String DEFAULT_BINDIR = "bin";
	public static final String DEFAULT_COMMAND = "selene";
	public static final String DEFAULT_DOCDIR = "src";

	private static final long serialVersionUID = -3348254851719601738L;
	private static final int HANDLE_NOT_FOUND = 3;
	private static final int NO_INPUT = 7;
	private static final int NO_MATCH = 9;

	public static String config = ".selene";
	private String root;
	private String bindir;
	private String command;
	private String docdir;
	private String exec;

	private String repository;
	private Process p;
	private InstantBufferedReader in;
	private BufferedReader err;
	private PrintStream out;

	public SeleneServer() throws RemoteException, IOException {
//		super();
		super(1100);
		String home = System.getProperty("user.home");
		Properties options = new Properties();
		options.load(new FileInputStream(home + prependSeparator(config)));
		root = options.getProperty("root", home);
		bindir = prependSeparator(
			options.getProperty("bindir", DEFAULT_BINDIR));
		command = prependSeparator(
			options.getProperty("command", DEFAULT_COMMAND));
		docdir = prependSeparator(
			options.getProperty("docdir", DEFAULT_DOCDIR));
		exec = root + bindir + command;
	}

	public static String prependSeparator(String path) {
		return path == "" || path == null ? path : File.separator + path;
	}

	public synchronized String search(String repository, String document,
		int cursorPosition, double attCoeff, double threshold)
		throws IOException {
		if (this.repository == null || !this.repository.equals(repository)) {
			if (p != null) p.destroy();
			p = Runtime.getRuntime().exec(
				new String[] {exec, repository, "-c"});
			in = new InstantBufferedReader(new InputStreamReader(
				p.getInputStream()));
			err = new BufferedReader(
				new InputStreamReader(p.getErrorStream()));
			out = new PrintStream(p.getOutputStream(), true);

			new Thread() {
				{ start(); }

				public void run() {
					String line;
					try {
						while ((line = err.readLine()) != null)
							System.out.println(line);
					} catch (IOException e) {
						System.out.println(
							"SeleneServer error: " + e.getMessage());
					}
				}
			};
		}

		byte[] encodedDocument = document.getBytes();
		out.println(encodedDocument.length +
			" " + cursorPosition + " " + attCoeff + " " + threshold);
		out.write(encodedDocument);

		String size = in.readLine(), result = "";
		if (size != null) {
			result = in.readTo(Integer.parseInt(size));
		}

		this.repository = null;
		try {
			int code;
			switch (code = p.exitValue()) {
				case 0:
					break;
				case HANDLE_NOT_FOUND:
					throw new FileNotFoundException
						("Repository \"" + repository + "\" does not exist");
				case NO_INPUT:
					break;
				case NO_MATCH:
					break;
				default:
					throw new RemoteException(
						"Search engine internal error #" + code);
			}
		} catch (IllegalThreadStateException e) {
			this.repository = repository;
		}
		return result;
	}

	public String getDocument(String repository, String name)
		throws IOException {
		String document = root + docdir +
			prependSeparator(repository) + prependSeparator(name);
		InstantBufferedReader in =
			new InstantBufferedReader(new FileReader(document));
		String result = in.readToEnd();
		in.close();
		return result;
	}

	public static void main(String[] args) {
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}

		try {
			SeleneServer obj = new SeleneServer();
			Naming.rebind("SeleneServer", obj);
			System.out.println("SeleneServer bound in registry");
		} catch (Exception e) {
			System.out.println("SeleneServer error: " + e.getMessage());
		}
	}
}
